==> rules-0.6-20150422.133437-1.jar: rules/core.clj <==
(ns rules.core
  (:require [datomic.api :refer [q]]
            [clojure.core.match :refer [match]]))

(defn rule-match?
  "rule is a vector of vectors. First vector must have a ?variable at the end
  user-data is user stats from the db
  unifier is a symbol like ?id

  Keyword->Rule->[UserStat]->Boolean"
  [unifier rule user-data]
  (when (q (concat [:find [unifier]
                    :where]
                   rule)
           user-data)
    true))

(defn rules-result
  "DB->CampaignMap->[UserStat]->[Rule]->CampaignId
  Returns original campaign id if nothing matches
  [Rule] should be the list found under keys like :history or :network in rules.edn

  NOTE: UserStat MUST be in a vector of vectors form"
  [campaign-id rules user-data]
  (if-let [dest-id (first (filter identity (for [{:keys [unifier clauses result]} rules]
                                             (if (rule-match? unifier clauses user-data)
                                               result nil))))]
    dest-id
    campaign-id))

;;RULE VALIDATION
(defn valid-clauses?
  [clauses]
  (every? identity
          (for [c clauses]
            (match [c]
                   [[(_ :guard integer?) (_ :guard keyword?) _ (_ :guard symbol?)]] true
                   [[_]] true
                   :else     nil))))

(defn valid-rule?
  [rule]
  (match [rule]
         [{:name (_ :guard string?) :result (n :guard integer?) :unifier (_ :guard symbol?)
           :clauses (cs :guard vector?)}] (valid-clauses? cs)
           :else nil))

(defn valid-rules-campaign?
  "must have :history and/or :network as keys"
  [rc]
  (match [rc]
         [({:history rules})]
         (every? identity (map valid-rule? rules))
         [({:network rules})]
         (every? identity (map valid-rule? rules))
         :else nil))

(defn valid-rules-map?
  [rules-map]
  (every? identity
          (for [[camp-id rules-campaign] rules-map]
            (and
             (integer? camp-id)
             (valid-rules-campaign? rules-campaign)))))

==> rules-0.6-20150422.140129-2.jar: rules/core.clj <==
(ns rules.core
  (:require [datomic.api :refer [q]]
            [clojure.core.match :refer [match]]))

(defn rule-match?
  "rule is a vector of vectors. First vector must have a ?variable at the end
  user-data is user stats from the db
  unifier is a symbol like ?id

  Keyword->Rule->[UserStat]->Boolean"
  [unifier rule user-data]
  (when (q (concat [:find [unifier]
                    :where]
                   rule)
           user-data)
    true))

(defn rules-result
  "DB->CampaignMap->[UserStat]->[Rule]->CampaignId
  Returns original campaign id if nothing matches
  [Rule] should be the list found under keys like :history or :network in rules.edn

  NOTE: UserStat MUST be in a vector of vectors form"
  [campaign-id rules user-data]
  (if-let [dest-id (first (filter identity (for [{:keys [unifier clauses result]} rules]
                                             (if (rule-match? unifier clauses user-data)
                                               result nil))))]
    dest-id
    campaign-id))

;;RULE VALIDATION
(defn valid-clauses?
  [clauses]
  (every? identity
          (for [c clauses]
            (match [c]
                   [[(_ :guard integer?) (_ :guard keyword?) _ (_ :guard symbol?)]] true
                   [[_]] true
                   :else     nil))))

(defn valid-rule?
  [rule]
  (match [rule]
         [{:name (_ :guard string?) :result (n :guard integer?) :unifier (_ :guard symbol?)
           :clauses (cs :guard vector?)}] (valid-clauses? cs)
           :else nil))

(defn valid-rules-campaign?
  "must have :history and/or :network as keys"
  [rc]
  (match [rc]
         [{:history rules}]
         (every? identity (map valid-rule? rules))
         [{:network rules}]
         (every? identity (map valid-rule? rules))
         :else nil))

(defn valid-rules-map?
  [rules-map]
  (let [checked-rules (for [[camp-id rules-campaign] rules-map]
                        [camp-id
                         (and
                          (integer? camp-id)
                          (valid-rules-campaign? rules-campaign))])]
    (if (every? second checked-rules)
      true
      (do (println {:bad-rules (vec (->> (filter (comp not second) checked-rules)
                                         (map first)))})
          nil))))
